package ch.phoenix.oss.quarkus.commons.tracing;

import static org.junit.jupiter.params.provider.Arguments.arguments;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.ArgumentMatchers.startsWith;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;

import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.mockito.InjectSpy;
import io.restassured.RestAssured;
import io.restassured.http.ContentType;
import java.util.Map;
import java.util.stream.Stream;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

@QuarkusTest
class RoutePatternTest {

    @InjectSpy
    TracingService tracingService;

    static Stream<Arguments> get() {
        return Stream.of(
                arguments("/", Map.of()),
                arguments("/leading-and-no-trailing", Map.of()),
                arguments("/leading/{param}/{param2}", Map.of("param", "1", "param2", "2")),
                arguments("/{param}/{param2}/trailing", Map.of("param", "1", "param2", "2")),
                arguments("/leading-and-no-trailing/{param}", Map.of("param", "1")),
                arguments("/leading-and-no-trailing/{param}/{param2}", Map.of("param", "1", "param2", "2")),
                arguments("/leading-and-trailing", Map.of()),
                arguments("/leading-and-trailing/{param}", Map.of("param", "1")),
                arguments("/leading-and-trailing/{param}/{param2}", Map.of("param", "1", "param2", "2")),
                arguments("/no-leading-and-no-trailing", Map.of()),
                arguments("/no-leading-and-no-trailing/{param}", Map.of("param", "1")),
                arguments("/no-leading-and-no-trailing/{param}/{param2}", Map.of("param", "1", "param2", "2")),
                arguments("/no-leading-and-trailing", Map.of()),
                arguments("/no-leading-and-trailing/{param}", Map.of("param", "1")),
                arguments("/no-leading-and-trailing/{param}/{param2}", Map.of("param", "1", "param2", "2")));
    }

    @MethodSource
    @ParameterizedTest
    void get(String route, Map<String, String> pathParams) {
        RestAssured.given()
                .accept(ContentType.TEXT)
                .when()
                .get(route, pathParams)
                .then()
                .statusCode(200);

        verify(tracingService).trace("actor", "anonymous");
        verify(tracingService).trace("request.method", "GET");
        verify(tracingService).trace("request.route", route);
        pathParams.forEach((key, value) -> verify(tracingService).trace("request.path.params." + key, value));
        verify(tracingService).trace("request.headers.accept", "text/plain");
        verify(tracingService).trace("request.headers.accept-encoding", "gzip,deflate");
        verify(tracingService).trace("request.headers.connection", "Keep-Alive");
        verify(tracingService).trace(eq("request.headers.host"), startsWith("localhost:"));
        verify(tracingService).trace(eq("request.headers.user-agent"), startsWith("Apache-HttpClient"));
        verify(tracingService).trace("request.client.ip", "127.0.0.1");
        verifyNoMoreInteractions(tracingService);
    }

    @Test
    void postBlankResource() {
        var route = "/";
        RestAssured.given().accept(ContentType.TEXT).when().post(route).then().statusCode(200);

        verify(tracingService).trace("actor", "anonymous");
        verify(tracingService).trace("request.method", "POST");
        verify(tracingService).trace("request.route", route);
        verify(tracingService).trace("request.headers.accept", "text/plain");
        verify(tracingService).trace("request.headers.accept-encoding", "gzip,deflate");
        verify(tracingService).trace("request.headers.connection", "Keep-Alive");
        verify(tracingService).trace("request.headers.content-length", "0");
        verify(tracingService)
                .trace("request.headers.content-type", "application/x-www-form-urlencoded; charset=ISO-8859-1");
        verify(tracingService).trace(eq("request.headers.host"), startsWith("localhost:"));
        verify(tracingService).trace(eq("request.headers.user-agent"), startsWith("Apache-HttpClient"));
        verify(tracingService).trace("request.client.ip", "127.0.0.1");
        verifyNoMoreInteractions(tracingService);
    }
}
